
package com.ibm.oti.util;

/*
 * Licensed Materials - Property of IBM,
 * (c) Copyright IBM Corp. 1998, 2003  All Rights Reserved
 */

import java.security.AccessController;
import java.security.PrivilegedAction;

/**
 * This class retrieves strings from a resource bundle
 * and returns them, formatting them with MessageFormat
 * when required.
 * <p>
 * It is used by the system classes to provide national
 * language support, by looking up messages in the
 * <code>
 *    com.ibm.oti.util.ExternalMessages
 * </code>
 * resource bundle. Note that if this file is not available,
 * or an invalid key is looked up, or resource bundle support
 * is not available, the key itself will be returned as the
 * associated message. This means that the <em>KEY</em> should
 * a reasonable human-readable (english) string.
 *
 * @author		OTI
 * @version		initial
 */

import java.util.*;

public class Msg {

	// ResourceBundle holding the system messages.
	static private ResourceBundle bundle = null;

	static {
	  // Attempt to load the messages.
		setLocale(Locale.getDefault());
	}

/**
 * Retrieves a message which has no arguments.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		msg String
 *					the key to look up.
 * @return		String
 *					the message for that key in the system
 *					message bundle.
 */
static public String getString (String msg) {
	if (bundle == null)
		return msg;
	try {
		return bundle.getString(msg);
	} catch (MissingResourceException e) {
		return msg;
	}
}

/**
 * Retrieves a message which takes 1 argument.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		msg String
 *					the key to look up.
 * @param		arg Object
 *					the object to insert in the formatted output.
 * @return		String
 *					the message for that key in the system
 *					message bundle.
 */
static public String getString (String msg, Object arg) {
	return getString(msg, new Object[] {arg});
}

/**
 * Retrieves a message which takes 1 integer argument.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		msg String
 *					the key to look up.
 * @param		arg int
 *					the integer to insert in the formatted output.
 * @return		String
 *					the message for that key in the system
 *					message bundle.
 */
static public String getString (String msg, int arg) {
	return getString(msg, new Object[] {Integer.toString(arg)});
}

/**
 * Retrieves a message which takes 1 character argument.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		msg String
 *					the key to look up.
 * @param		arg char
 *					the character to insert in the formatted output.
 * @return		String
 *					the message for that key in the system
 *					message bundle.
 */
static public String getString (String msg, char arg) {
	return getString(msg, new Object[] {String.valueOf(arg)});
}

/**
 * Retrieves a message which takes 2 arguments.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		msg String
 *					the key to look up.
 * @param		arg1 Object
 *					an object to insert in the formatted output.
 * @param		arg2 Object
 *					another object to insert in the formatted output.
  * @return		String
 *					the message for that key in the system
 *					message bundle.
 */
static public String getString (String msg, Object arg1, Object arg2) {
	return getString(msg, new Object[] {arg1, arg2});
}

/**
 * Retrieves a message which takes several arguments.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		msg String
 *					the key to look up.
 * @param		args Object[]
 *					the objects to insert in the formatted output.
 * @return		String
 *					the message for that key in the system
 *					message bundle.
 */
static public String getString (String msg, Object[] args) {
	String format = msg;

	if (bundle != null) {
		try {
			format = bundle.getString(msg);
		} catch (MissingResourceException e) {}
	}

	return format(format, args);
}

/**
 * Generates a formatted text string given a source string
 * containing "argument markers" of the form "{argNum}"
 * where each argNum must be in the range 0..9. The result
 * is generated by inserting the toString of each argument
 * into the position indicated in the string.
 * <p>
 * To insert the "{" character into the output, use a single
 * backslash character to escape it (i.e. "\{"). The "}"
 * character does not need to be escaped.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		format String
 *					the format to use when printing.
 * @param		args Object[]
 *					the arguments to use.
 * @return		String
 *					the formatted message.
 */
static private String format (String format, Object[] args) {
	StringBuffer answer = new StringBuffer();
	String[] argStrings = new String[args.length];
	for (int i = 0; i < args.length; ++i) {
		if (args[i] == null)
			argStrings[i] = "<null>";
		else
			argStrings[i] = args[i].toString();
	}
	int lastI = 0;
	for (int i = format.indexOf('{', 0); i >= 0; i = format.indexOf('{', lastI)) {
		if (i != 0 && format.charAt(i-1) == '\\') {
			// It's escaped, just print and loop.
			if (i != 1)
				answer.append(format.substring(lastI,i-1));
			answer.append('{');
			lastI = i+1;
		} else {
			// It's a format character.
			if (i > format.length()-3) {
				// Bad format, just print and loop.
				answer.append(format.substring(lastI, format.length()));
				lastI = format.length();
			} else {
				int argnum = (byte) Character.digit(format.charAt(i+1), 10);
				if (argnum < 0 || format.charAt(i+2) != '}') {
					// Bad format, just print and loop.
					answer.append(format.substring(lastI, i+1));
					lastI = i+1;
				} else {
					// Got a good one!
					answer.append(format.substring(lastI, i));
					if (argnum >= argStrings.length)
						answer.append("<missing argument>");
					else
						answer.append(argStrings[argnum]);
					lastI = i + 3;
				}
			}
		}
	}
	if (lastI < format.length())
		answer.append(format.substring(lastI, format.length()));
	return answer.toString();
}
/**
 * Changes the locale of the messages.
 *
 * @author		OTI
 * @version		initial
 *
 * @param		locale Locale
 *					the locale to change to.
 */
static public void setLocale (final Locale locale) {
	bundle = null;
	try {
		bundle = (ResourceBundle)AccessController.doPrivileged(new PrivilegedAction() {
			public Object run() {
				return ResourceBundle.getBundle("com.ibm.oti.util.ExternalMessages", locale, com.ibm.oti.vm.VM.getStackClassLoader(1));
			}});
	} catch (MissingResourceException e) {}
}

}
